#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <util/delay.h>

// SPI ----------------------------------------------------
#define SPI_MOSI		6		// SPI Master Output
#define SPI_MISO		5		// SPI Master Input
#define SPI_SCLK		4		// SPI Serial Clock
#define SPI_nCS			1		// SPI Chip Select
// --------------------------------------------------------

// Oscillators Enable Signals -----------------------------
#define EN_x44			2		// PA2
#define EN_x48			1		// PA1
// --------------------------------------------------------

// Select Oscillator Signal -------------------------------
#define SEL_OSC			2		// PB2
// --------------------------------------------------------

// PCM Reset Signal ---------------------------------------
#define PCM_RST			0		// PB0
// --------------------------------------------------------

// PCM Registers ------------------------------------------
#define READ			1
#define WRITE			0

#define PCM_REG16		0
#define PCM_REG17		1
#define PCM_REG18		2
#define PCM_REG19		3
#define PCM_REG20		4
#define PCM_REG21		5
#define PCM_REG22		6
#define PCM_REG23		7
// --------------------------------------------------------

// Register 18 --------------------------------------------
#define MUTE			0
#define DME				1
#define DMF0			2
#define DMF1			3
#define FMT0			4
#define FMT1			5
#define FMT2			6
#define ATLD			7
// --------------------------------------------------------

// Register 19 --------------------------------------------
#define INZD			0
#define FLT				1
#define DFMS			2
// Bit 3 Reserved
#define OPE				4
#define ATS0			5
#define ATS1			6
#define REV				7
// --------------------------------------------------------

// Register 20 --------------------------------------------
#define OS0				0
#define OS1				1
#define CHSL			2
#define MONO			3
#define DFTH			4
#define DSD				5
#define SRST			6
// Bit 7 Reserved
// --------------------------------------------------------

// Register 21 --------------------------------------------
#define PCMZ_BIT		0
#define DZ0_BIT			1
#define DZ1_BIT			2
// Bits 3-7 Reserved
// --------------------------------------------------------

// Register 22 --------------------------------------------
#define ZFGL_BIT		0
#define ZFGR_BIT		1
// Bits 2-7 Reserved
// --------------------------------------------------------

#define UNMUTE_DELAY	100								//   MUTE,  

volatile unsigned char osc_change_flag = 1;				//    (  1,      )

void init_system(void)
{
	//------------------------------------------------------------------------
	//   
	//------------------------------------------------------------------------
		
	// SPI
	DDRA |= (1 << SPI_MOSI) | (1 << SPI_SCLK);
	DDRB |= (1 << SPI_nCS);
	PORTB |= (1 << SPI_nCS);
	
	DDRA &= ~(1 << SPI_MISO);	
	
	//    
	DDRA |= (1 << EN_x44) | (1 << EN_x48);
	PORTA &= ~((1 << EN_x44) | (1 << EN_x48));
	
	//   
	DDRB &= ~(1 << SEL_OSC);
	
	//   PCM
	DDRB |= (1 << PCM_RST);
	PORTB |= (1 << PCM_RST);
	
	//------------------------------------------------------------------------
	//   Pin Change Interrupts
	//------------------------------------------------------------------------
	
	GIMSK |= (1 << PCIE1);								// Pin Change Interrupt Enable 1
	PCMSK1 |= (1 << PCINT10);							// Pin Change 10 Enable
	
	//------------------------------------------------------------------------
	//  SPI
	//------------------------------------------------------------------------
	
	//USICR |= (1 << USIWM0);								// Three-wire mode. Uses DO, DI, and USCK pins.
	//USICR |= (1 << USICLK) | (1 << USICS1);				// External Clock Source, positive edge & Software clock strobe (USITC)

	//------------------------------------------------------------------------
	//   
	//------------------------------------------------------------------------
	
	sleep_enable();										//    
	set_sleep_mode(SLEEP_MODE_PWR_DOWN);				//     Power-Down
}

unsigned char SPI_receive(unsigned char addr_reg)
{
	PORTB &= ~(1 << SPI_nCS);							//  
	
	unsigned char send_data = 0, receive_data = 0;
	
	send_data = (0b10010000 | addr_reg);				
	
	for (unsigned char i = 0; i < 8; i++)
	{
		// 
		if ((send_data & 128) == 0)
		{
			PORTA &= ~(1 << SPI_MOSI);
		}
		else
		{
			PORTA |= (1 << SPI_MOSI);
		}
		
		send_data = send_data << 1;
		
		// 
		PORTA |= (1 << SPI_SCLK);
		receive_data = receive_data << 1;
		if ((PINA & (1 << SPI_MISO)) != 0x00)
		{
			receive_data = receive_data | 0x01;
		}
		
		PORTA &= ~(1 << SPI_SCLK);
	}
	
	send_data = 0;
	
	for (unsigned char i = 0; i < 8; i++)
	{
		// 
		if ((send_data & 128) == 0)
		{
			PORTA &= ~(1 << SPI_MOSI);
		}
		else
		{
			PORTA |= (1 << SPI_MOSI);
		}
		
		send_data = send_data << 1;
		
		// 
		PORTA |= (1 << SPI_SCLK);
		receive_data = receive_data << 1;
		if ((PINA & (1 << SPI_MISO)) != 0x00)
		{
			receive_data = receive_data | 0x01;
		}
		
		PORTA &= ~(1 << SPI_SCLK);
	}
	
	PORTB |= (1 << SPI_nCS);							//  
	
	return receive_data;
}

void SPI_transmit(unsigned char addr_reg, unsigned char data_reg)
{	
	PORTB &= ~(1 << SPI_nCS);							//  
	
	unsigned char send_data = 0, receive_data = 0;
	
	send_data = (0b00010000 | addr_reg);
	
	for (unsigned char i = 0; i < 8; i++)
	{
		// 
		if ((send_data & 128) == 0)
		{
			PORTA &= ~(1 << SPI_MOSI);
		}
		else
		{
			PORTA |= (1 << SPI_MOSI);
		}
		
		send_data = send_data << 1;
		
		// 
		PORTA |= (1 << SPI_SCLK);
		receive_data = receive_data << 1;
		if ((PINA & (1 << SPI_MISO)) != 0x00)
		{
			receive_data = receive_data | 0x01;
		}
		
		PORTA &= ~(1 << SPI_SCLK);
	}
	
	send_data = data_reg;
	
	for (unsigned char i = 0; i < 8; i++)
	{
		// 
		if ((send_data & 128) == 0)
		{
			PORTA &= ~(1 << SPI_MOSI);
		}
		else
		{
			PORTA |= (1 << SPI_MOSI);
		}
		
		send_data = send_data << 1;
		
		// 
		PORTA |= (1 << SPI_SCLK);
		receive_data = receive_data << 1;
		if ((PINA & (1 << SPI_MISO)) != 0x00)
		{
			receive_data = receive_data | 0x01;
		}
		
		PORTA &= ~(1 << SPI_SCLK);
	}
	
	PORTB |= (1 << SPI_nCS);							//  
}

/*
unsigned char SPI_receive(unsigned char addr_reg)
{
	unsigned char temp = 0;
	
	PORTB &= ~(1 << SPI_nCS);							//  
	
	USIDR = (0b10010000 | addr_reg);					//     USI
	USISR |= (1 << USIOIF);								//    
	
	//   8 
	while((USISR & (1 << USIOIF)) == 0)
	{
		USICR |= ((1 << USIWM0) | (1 << USICS1) | (1 << USICLK) | (1 << USITC));
	}
	
	USISR |= (1 << USIOIF);								//    
	
	//   8 
	while((USISR & (1 << USIOIF)) == 0)
	{
		USICR |= ((1 << USIWM0) | (1 << USICS1) | (1 << USICLK) | (1 << USITC));
	}
	
	temp = USIDR;										//   
	
	PORTB |= (1 << SPI_nCS);							//  
	
	return temp;
}

void SPI_transmit(unsigned char addr_reg, unsigned char data_reg)
{
	PORTB &= ~(1 << SPI_nCS);							//  
	
	USIDR = (0b00010000 | addr_reg);					//     USI
	USISR |= (1 << USIOIF);								//    
	
	//   8 
	while((USISR & (1 << USIOIF)) == 0)
	{
		USICR |= ((1 << USIWM0) | (1 << USICS1) | (1 << USICLK) | (1 << USITC));
	}
	
	USIDR = data_reg;									//     USI
	
	USISR |= (1 << USIOIF);								//    
	
	//   8 
	while((USISR & (1 << USIOIF)) == 0)
	{
		USICR |= ((1 << USIWM0) | (1 << USICS1) | (1 << USICLK) | (1 << USITC));
	}
	
	PORTB |= (1 << SPI_nCS);							//  
}*/

void PCM_init(void)
{
	unsigned char data = 0;
	
	data = SPI_receive(PCM_REG19);						//   19- 
	data |= (1 << ATS1);								//  
	SPI_transmit(PCM_REG19, data);						//   
}

void PCM_mute_ON(void)
{
	unsigned char data = 0;
	
	data = SPI_receive(PCM_REG18);						//   18- 
	data |= (1 << MUTE);								//   MUTE
	SPI_transmit(PCM_REG18, data);						//   
}

void PCM_mute_OFF(void)
{
	unsigned char data = 0;
	
	data = SPI_receive(PCM_REG18);						//   18- 
	data &= ~(1 << MUTE);								//   MUTE
	SPI_transmit(PCM_REG18, data);						//   
}

ISR(PCINT1_vect)
{
	osc_change_flag = 1;
}

int main(void)
{
    init_system();							
	sei();												//   
	PCM_mute_ON();										//  MUTE
	PCM_init();											//      
	
	while (1) 
    {
		if (osc_change_flag)
		{
			osc_change_flag = 0;						//  
			
			PCM_mute_ON();								//  MUTE
			
			//       SEL_OSC
			if (PINB & (1 << SEL_OSC))
			{
				//    x44
				PORTA &= ~(1 << EN_x48);
				PORTA |= (1 << EN_x44);
			}
			else
			{
				//    x48
				PORTA &= ~(1 << EN_x44);
				PORTA |= (1 << EN_x48);
			}
			
			_delay_ms(UNMUTE_DELAY);
			PCM_mute_OFF();								//  MUTE
		}
		
		if (!(osc_change_flag))
		{
			sleep_cpu();								//    
		}
		
    }
}